@@ -33,6 +33,9 @@
 #include "telepathy-glib/debug-internal.h"
 #include "telepathy-glib/proxy-internal.h"
 
+/* channel-group.c is ~all deprecated APIs, modern APIs are in
+ * channel-contacts.c. So we allow this module to use deprecated functions. */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 
 /**
  * TP_ERRORS_REMOVED_FROM_GROUP:
@@ -79,6 +82,7 @@ local_pending_info_free (LocalPendingInfo *info)
  *
  * Returns: the handle representing the user, or 0
  * Since: 0.7.12
+ * Deprecated: New code should use tp_channel_group_get_self_contact() instead.
  */
 TpHandle
 tp_channel_group_get_self_handle (TpChannel *self)
@@ -122,6 +126,8 @@ tp_channel_group_get_flags (TpChannel *self)
  *
  * Returns: (transfer none): the members, or %NULL
  * Since: 0.7.12
+ * Deprecated: New code should use tp_channel_group_dup_members_contacts()
+ *  instead.
  */
 const TpIntset *
 tp_channel_group_get_members (TpChannel *self)
@@ -146,6 +152,8 @@ tp_channel_group_get_members (TpChannel *self)
  *
  * Returns: (transfer none): the local-pending members, or %NULL
  * Since: 0.7.12
+ * Deprecated: New code should use tp_channel_group_dup_local_pending_contacts()
+ *  instead.
  */
 const TpIntset *
 tp_channel_group_get_local_pending (TpChannel *self)
@@ -170,6 +178,8 @@ tp_channel_group_get_local_pending (TpChannel *self)
  *
  * Returns: (transfer none): the remote-pending members, or %NULL
  * Since: 0.7.12
+ * Deprecated: New code should use
+ *  tp_channel_group_dup_remote_pending_contacts() instead.
   */
 const TpIntset *
 tp_channel_group_get_remote_pending (TpChannel *self)
@@ -203,6 +213,8 @@ tp_channel_group_get_remote_pending (TpChannel *self)
  *
  * Returns: %TRUE if the contact is in fact local-pending
  * Since: 0.7.12
+ * Deprecated: New code should use
+ *  tp_channel_group_get_local_pending_contact_info() instead.
  */
 gboolean
 tp_channel_group_get_local_pending_info (TpChannel *self,
@@ -302,6 +314,7 @@ tp_channel_group_get_local_pending_info (TpChannel *self,
  *
  * Returns: the global handle that owns the given handle, or 0
  * Since: 0.7.12
+ * Deprecated: New code should use tp_channel_group_get_contact_owner() instead.
  */
 TpHandle
 tp_channel_group_get_handle_owner (TpChannel *self,
@@ -960,7 +973,7 @@ _tp_channel_group_improve_remove_error (TpChannel *self,
 
   /* If we changed the code we also need to change the domain; if not, we did
    * an early return, so we'll never reach this */
-  error->domain = TP_ERRORS;
+  error->domain = TP_ERROR;
 }
 
 static void
@@ -1385,3 +1398,5 @@ _tp_channel_get_group_properties (TpChannel *self)
       TP_IFACE_CHANNEL_INTERFACE_GROUP, tp_channel_got_group_properties_cb,
       NULL, NULL, NULL);
 }
+
+G_GNUC_END_IGNORE_DEPRECATIONS
